(**************************************************************************)
(*                                                                        *)
(* PXDUMP - Command line utility to export contents of Paradox table.     *)
(*          This is freeware with source                                  *)
(*                                                                        *)
(* SYNOPSIS                                                               *)
(* The command line should read "PxDump pxtable.db" or "PxDump pxtable".  *)
(* The contents of the Paradox table named "pxtable.db" will be dumped to *)
(* an ASCII CSV file called "pxtable.txt" residing in the same directory  *)
(*                                                                        *)
(* AUTHOR:  Mario Pinkster  ( dimario@freeemail.nl)                       *)
(* VERSION: 1.0                                                           *)
(* DATE :   Januari 2, 2000 AD                                            *)
(**************************************************************************)

program PxDump;

{$APPTYPE CONSOLE}

uses
  DBTables,
  IniFiles,
  SysUtils;

{ NOTE: In order to get a smaller footprint for the executable, experiment
  somewhat with including runtime packages.
  For Delphi 4:
  In the Project Options, Packages tab, I checked the option "build
  with runtime packages". Then I removed the default runtime packages
  list and added "DclDB40" as the only runtime package. When compiled
  without debug information this results in an executable of 10240 bytes.
}

{  ExportASCII
   Exports the contents of a Paradox table to a CSV file format
   The resulting ASCII file has the same name as the
   table, but an extention of '*.txt'
   The variable aFile should contain a full pathname for a Paradox
   file. The extention '.db' is optional
}

procedure ExportASCII(const aFile: string);
var
  tbl                 : TTable;
  ascii               : TTable;
  ini                 : TIniFile;
  section             : string;
begin
  // Make vars explicitly nil so they are freed correctly in case of exception.
  tbl := nil;
  ascii := nil;
  ini := nil;
  try
    // Create the input table object, assuming it is a Paradox table
    tbl := TTable.Create(nil);
    tbl.TableName := ExtractFileName(aFile);
    tbl.DatabaseName := ExtractFilePath(aFile);
    tbl.TableType := ttParadox;
    // Open the input table
    tbl.Open;

    // Create the output ASCII file object
    ascii := TTable.Create(nil);
    ascii.TableName := ExtractFileName(aFile);
    ascii.DatabaseName := ExtractFilePath(aFile);
    ascii.FieldDefs.Clear;
    ascii.FieldDefs.Assign(tbl.FieldDefs);
    ascii.TableType := ttASCII;

    // Create the output ASCII file itself
    // This call will produce a zero-length output file plus a schema file
    ascii.CreateTable;

    // The default export method for an ASCII file is fixed record length.
    // In order to obtain neat comma-separated values (CSV) we need
    // to change some things in the schema file.
    // The schema file has the same name as the ASCII file but with
    // extention '.sch'. It also lives in the same directory.

    // The schema file is organized as an INI file and contains a
    // section named after the root file name (no path, no extention)
    // of the ASCII file. This section must be changed to supply these
    // these values:

    //  FILETYPE=DELIMITED (Default filetype is Fixed)
    //  DELIMITER=" (by default, this setting is missing from the schema file)
    //  SEPARATOR=, (by default, this setting is missing from the schema file)
    //  CHARSET=ASCII (This is already so by default)

    //  One last piece of information: according to several sources I found
    //  on DejaNews, the batchmove _must_ be performed in batAPPEND mode.

    // Open the schema file as a regular INI file
    ini := TIniFile.Create(ChangeFileExt(aFile, '.SCH'));

    // Extract the name of the section we are going to attack
    section := UpperCase(ExtractFileName(ChangeFileExt(aFile, '')));

    // Write the information we want the schema to contain
    ini.WriteString(section, 'FILETYPE', 'DELIMITED');
    ini.WriteString(section, 'CHARSET', 'ASCII');
    ini.WriteString(section, 'DELIMITER', '"');
    ini.WriteString(section, 'SEPARATOR', ',');

    // Perform the batchmove from Paradox table to ASCII CSV file
    ascii.BatchMove(tbl, batAppend);
  finally
    // Do some cleaning up. In case of exceptions, it is no problem
    // when variables that are still nil are freed.
    tbl.Free;
    ascii.Free;
    ini.Free;

    // Optionally, clean up the schema file
    // DeleteFile( ChangeFileExt( aFile, '.SCH'));
  end;
end;

// Main entry point of the program
var
  s                   : string;
begin
  // See if we have an argument to process
  s := ParamStr(1);
  if Length(s) > 0 then begin
    // Yup. Put full path to this file in the call to the worker function
    try
      ExportASCII(GetCurrentDir + '\' + ChangeFileExt(s, ''));
    except
      // In case of problems, try to print some information before dying
      on E: Exception do begin
        WriteLn(E.Message);
        Halt(1);
      end;
    end;
  end;
end.

